;============================================================================
;   Programme  : Serial Communication (Light)                
;   -version   : 1.2
;   -fonction  : communication par port de serie entre 2 ordinateurs
;   -caractristiques: 
;        + test de la presence, et l'activation de FIFO sur 16550
;        + configuration logicielle des ports
;        + prise en charge du signal BREAK
;   -date          : le 8 Fev 2002 (v1.0 )
;   -historique    : 
;        + le 25 Sep 2002 -> adaptation du format COM au EXE (v1.1)
;        + le 13 Nov 2002 -> configuration logicielle des ports (v1.2)
;                    
;   -environnement : DOS 16 bits ou console DOS 32 bits
;   -compilateur   : TASM et TLINK de Borland
;   -directives de compilation :
;          TASM 8250fin
;          TLINK 8250fin
;   -auteur: Duong-Khang NGUYEN - email: monsieur_neurone@hotmail.com
;============================================================================
Segment _STACK STACK 'STACK' word
  LaPile dw 30 dup (0ABCDh)  ;-- pile de 60 octets = largement suffisant  
Ends _STACK

Segment _DATA word
  ;----------------- constantes --------------------
  CRLF equ 0Ah,0Dh
  COM1 = 03F8h
  COM2 = 02F8h
  VRAI    = 1
  FAUX    = 0

  ;----------------- variables ------------------------
  portxxDLu   db FAUX

  portxx9 dw ?         ;-- Interrupt Enable Register
  portxxA dw ?         ;-- Interrupt Identification Register
  portxxB dw ?         ;-- Line Control Register
  portxxC dw ?         ;-- Modem Control Register
  portxxD dw ?         ;-- Line Status Register

  err_cadence   dw offset msg_cadence
  err_trame     dw offset msg_trame
  err_parite    dw offset msg_parite

  ;----------------- messages ------------------------
  intro         db CRLF, 'Test de communication sur un UART'
                db CRLF, 'ralis par Duong-Khang NGUYEN'
                db CRLF, 'email: monsieur_neurone@hotmail.com','$'
  chip_16550    db CRLF, 'UART 16550 prsent, FIFO non utilisable dtect'
                db CRLF, 'changez d`UART vieux !','$'
  chip_absent   db CRLF, 'FIFO non prsent, acheter un UART 16550A !','$'
  chip_16550A   db CRLF, 'UART 16550A prsent avec FIFO de 16 octets','$'
  chip_16750    db CRLF, 'UART 16750 prsent avec FIFO de 64 octets','$'

  msg_FIFO         db CRLF, 'FIFO             ','$'
  msg_Break        db CRLF, 'Break            ','$'
  msg_LoopBack     db CRLF, 'Loopback         ','$'
  msg_Presentation db CRLF, 'Prsentation +   ','$'  
  msg_on           db '[ On  ]','$'
  msg_off          db '[ Off ]','$'

  baud_engage   db CRLF, '=================================================='
                db CRLF, 'Vitesse          ','[ OK  ]','$'
  config_paquet db CRLF, 'Trame            ','[ OK  ]','$'
  com_pret      db CRLF, 'Initialisation   ','[ OK  ]'
                db CRLF, '=================================================='
                db CRLF, 'Communication prte'
                db CRLF, CRLF, 'Appuyer sur une touche pour envoyer'
                db CRLF, 'ECHAP termine la communication',CRLF,'$'
  gen_break     db CRLF, 'Signal BREAK envoy','$'
  recep_break   db CRLF, 'Yo ! BREAK reu !','$'
  com_fin       db CRLF, 'Au revoir, a bientot','$'

  msg_distant   db CRLF, 'Elle -> ','$'
  msg_local     db CRLF, 'Moi  -> ','$'

  msg_cadence   db CRLF, ' err_cadence ','$'
  msg_trame     db CRLF, ' err_trame ','$'
  msg_parite    db CRLF, ' err_parite ','$'

  car_cadence   db '.','$'
  car_trame     db '?','$'
  car_parite    db '*','$'

  err_choix     db CRLF, '----------> opration annule','$'


  ;----------------------------------------------------------
  choix_menu    db CRLF, CRLF, 'Menu principal'
                db CRLF, '1. Configuration'
                db CRLF, '2. Configuration actuelle'
                db CRLF, '3. Go go go !'
                db CRLF, '0. Quitter'               
                db CRLF, CRLF, 'votre choix -> ','$'

  ;----------------------------------------------------------
  choix_config  db CRLF, CRLF, 'Menu configuration'
                db CRLF, '1. Changer le port COM'
                db CRLF, '2. Vitesse'
                db CRLF, '3. Longueur mot: nombre de bits, stop bit'
                db CRLF, '4. Parit'
                db CRLF, '5. Mode loopback'
                db CRLF, '6. FIFO'
                db CRLF, '7. Envoi et rception Break'
                db CRLF, '8. Prsentation'
                db CRLF, 'autres: annuler'
                db CRLF, 'Votre choix : ','$'

  ;----------------------------------------------------------
  choix_vitesse db CRLF, CRLF, 'Choix de la vitesse (en bauds)'
  label_vitesse db CRLF, '1. 1200   bauds' 
                db CRLF, '2. 2400   bauds'
                db CRLF, '3. 4800   bauds'
                db CRLF, '4. 9600   bauds'
                db CRLF, '5. 19200  bauds'
                db CRLF, '6. 38400  bauds'
                db CRLF, '7. 57600  bauds'
                db CRLF, '8. 115200 bauds'
                db CRLF, '9. ou autres: annuler'
                db CRLF, 'Votre choix : ','$'

  vitesse_COM   db 12       ;-- vitesse choisie par dfaut
  valeur_ref    db 96       ;-- valeur de reference pour 1200 bauds

  val_vitesse   db 3        ;-- pour affichage
  len_vitesse   db 17       ;-- longeur message

  ;----------------------------------------------------------
  choix_COM     db CRLF, CRLF, 'Choix de port COM'
  label_COM     db CRLF, '1. COM1 au port 3F8h'
                db CRLF, '2. COM2 au port 2F8h'
                db CRLF, '3 ou autres: annuler'
                db CRLF, 'Votre choix : ','$'

  COM           dw COM1     ;-- communication passe par COM1 par defaut
  val_COM       db 0
  len_COM       db 22

  ;----------------------------------------------------------
  choix_loopb   db CRLF, CRLF, 'Choix de Mode loop back (for testing purpose)'
                db CRLF, '1. loop back desactiv (par dfaut)'
                db CRLF, '2. loop back activ tout caractere mis est reu'
                db CRLF, 'autres: annuler'
                db CRLF, 'Votre choix : ','$'

  active_loopb  db FAUX     ;-- loopback dsactiv par dfaut

  ;----------------------------------------------------------
  choix_long    db CRLF, CRLF, 'Choix de longueur des mots'
  label_long    db CRLF, '1. 5 bits, 1 stop bit  '
                db CRLF, '2. 6 bits, 1 stop bit  '
                db CRLF, '3. 7 bits, 1 stop bit  '
                db CRLF, '4. 8 bits, 1 stop bit  '
                db CRLF, '5. 5 bits, 1.5 stop bit'
                db CRLF, '6. 6 bits, 2 stop bits '
                db CRLF, '7. 7 bits, 2 stop bits '
                db CRLF, '8. 8 bits, 2 stop bits '
                db CRLF, '9. ou autres: annuler'
                db CRLF, 'Votre choix : ','$'

  config_trame  db 00000011b     ;-- mode normal pour xx8 et xx9
                                 ;-- pas de parite
                                 ;-- 8 bits, 1 stop

  val_long      db 3
  len_long      db 25
  

  ;----------------------------------------------------------
  choix_parite  db CRLF, CRLF, 'Choix de parit'
  label_parite  db CRLF, '1. Pas de parit  (n)'
                db CRLF, '2. Parit impaire (o)'
                db CRLF, '3. Parit paire   (e)'
                db CRLF, '4. ou autres: annuler'
                db CRLF, 'Votre choix : ','$'
                                 ;--> voir config_trame en plus

  val_parite    db 0
  len_parite    db 23

  ;----------------------------------------------------------
  choix_FIFO    db CRLF, CRLF, 'Choix de tampon FIFO '
                db             '(UART ver. 16550A ou plus)'
                db CRLF, '1. test et activer FIFO 16 octets'
                db CRLF, '2. test et dsactiver FIFO'
                db CRLF, 'autres: annuler'
                db CRLF, 'Votre choix : ','$'

  active_FIFO   db VRAI          ;-- recherche FIFO et l'activer (par defaut)

  ;----------------------------------------------------------
  choix_break   db CRLF, CRLF, 'Choix de test du BREAK'
                db CRLF, '1. test et envoi de BREAK  la fin du programme '
                db CRLF, '2. dsactive l`envoi et test du BREAK'
                db CRLF, 'autres: annuler'
                db CRLF, 'Votre choix : ','$'

  control_break db VRAI          ;-- test de Break activ par dfaut

  ;----------------------------------------------------------
  choix_presentation db CRLF, CRLF, 'Choix de prsentation'
                db CRLF, '1. amliorer la prsentation (par dfaut)'
                db CRLF, '2. se conformer au sujet'
                db CRLF, 'autres: annuler'
                db CRLF, 'Votre choix : ','$'

  presentation_plus db VRAI
  distant_en_cours  db FAUX      ;-- machine distante est en cours de parler
  local_en_cours    db FAUX      ;-- machine locale   est en cours de parler
  nb_car_recu       db 0         ;-- nombre de caractres reus
  nb_car_emis       db 0
Ends _DATA


;============================== C O D E S ================================


Segment _CODE word
  assume CS:_CODE, DS:_DATA, SS:_STACK

introduction:
  Mov  AX, _DATA          ;-- initialisation du segment de donnees
  Mov  DS, AX

;  Mov  AX, _STACK         ;-- initialisation de la pile
;  Mov  SS, AX             ;-- PAS BESOIN, fait par le systme
;  LEA  SP, sommetPile     ;-- vrifi OK sous debug.exe

  LEA  DX, intro
  Mov  AH, 09h
  Int  21h                ;-- afficher message intro
  Jmp  debut


;============================================================================
;  procedure DesactiverIRQ
;    -entre: nant
;    -sortie: nant
;============================================================================
DesactiverIRQ proc near
  Mov  DX, portxxB         ;-- met le bit 7 a zero pour  
  Xor  AL, AL              ;-- desactiver les interruptions
  Out  DX, AL
  Mov  DX, portxx9
  Out  DX, AL              ;-- desactiver l'utilisation des interruptions
DIRQ_fin:
  ret
DesactiverIRQ endp


;============================================================================
;  procedure EngagerBAUD
;    -entre: nant
;    -sortie: nant
;============================================================================
EngagerBAUD proc near
  Mov  DX, portxxB
  Mov  AL, 10000000b       ;-- activer la configuration de Bauds
                           ;-- bit 7 sera RAZ par ConfigurerPaquet proc.
  Out  DX, AL
  Mov  DX, COM
  Mov  AL, vitesse_COM
  Out  DX, AL
  Xor  AL, AL
  Inc  DX                  ;-- = port xx9
  Out  DX, AL              ;-- 02, 00 = 56000 bauds
                           ;-- 03, 00 = 38400 bauds
                           ;-- 0C, 00 =  9600 bauds
EB_fin:
  LEA  DX, baud_engage
  Mov  AH, 09h
  Int  21h                 ;-- message succes
  ret
EngagerBAUD endp


;============================================================================
;  procedure ActiverFIFO
;    -entre: nant
;    -sortie: nant
;============================================================================
ActiverFIFO proc near
  Mov  DX, portxxA        ;-- write: FIFO Enable Register

  Mov  AL, 11000111b
  Out  DX, AL             ;-- bit 0: activer FIFO
                          ;-- bit 1: clear receive FIFO
                          ;-- bit 2: clear transmit FIFO
                          ;-- bit 6: = 1 and
                          ;-- bit 7: = 1 FIFO interruption sur
                          ;--          14 octets active
  In   AL, DX
  Mov  BL, AL
  And  BL, 11000000b      ;-- FIFO active ?
  Cmp  BL, 11000000b
  Je   COM_16550A

  Test AL, 10000000b      ;-- FIFO non utilisable ?
  Jnz  COM_16550

  Mov  AL, 11100111b
  Out  DX, AL             ;-- bit 0: activer FIFO
                          ;-- bit 1: clear receive FIFO
                          ;-- bit 2: clear transmit FIFO
                          ;-- bit 5: activer FIFO 64 octets
                          ;-- bit 6: = 1 and
                          ;-- bit 7: = 1 FIFO interruption sur
                          ;--          14 octets active
  In   AL, DX
  And  AL, 11000000b
  Cmp  AL, 11000000b
  Jne  FIFO_absent

  LEA  DX, chip_16750     ;-- wow 16750 present !
  Jmp  FIFO_demande

FIFO_Absent:
  LEA  DX, chip_absent
  Jmp  AFF_Fin

COM_16550A:
  LEA  DX, chip_16550A
  Jmp  FIFO_demande

COM_16550:
  LEA  DX, chip_16550

FIFO_demande:
  Mov  BX, DX             ;-- savegarder adresse message chip
  Test active_FIFO, VRAI
  Jnz  AFF_fin            ;-- demande d'activer FIFO, oui ? on sort

  Mov  AL, 11000110b
  Mov  DX, portxxA
  Out  DX, AL             ;-- bit 0 = 0: desactiver FIFO

AFF_fin:
  Mov  DX, BX             ;-- Reprise de l'adresse du message
  Mov  AH, 09h
  Int  21h
  ret
ActiverFIFO endp


;============================================================================
;  procedure ConfigurerPaquet
;    -entre: nant
;    -sortie: nant
;============================================================================
ConfigurerPaquet proc near
  Mov  DX, portxxB
  Mov  AL, config_trame    ;-- par defaut 8bits, pas de parite, 1 stop bit
  Out  DX, AL              ;-- break signal disabled
                           ;-- important ! remet en meme temps bit 7 a 0
                           ;-- pour pouvoir transmettre et recevoir
                           ;-- donnees sur portxxBase
CP_fin:
  LEA  DX, config_paquet
  Mov  AH, 09h
  Int  21h
  ret
ConfigurerPaquet endp


;============================================================================
;  procedure ComPret
;    -entre: nant
;    -sortie: nant
;============================================================================
ComPret proc near
  Mov  DX, portxxC
  Mov  AL, 00000000b       ;-- bit 0: Data Terminal Ready
  Test active_loopb, VRAI
  Jz   ComPret_Non_LoopBack
  Or   AL, 00010000b       ;-- bit 4 = 1: active loop back mode

ComPret_Non_LoopBack:

  Out  DX, AL              ;--     1: Request To Send
                           ;--     3: Interrupt Off
                           ;--     4: loop back mode
  Mov  DX, COM             ;-- RAZ port de reception
  In   AL, DX
  Mov  DX, portxxD         ;-- RAZ port etat de la ligne
  In   AL, DX

  LEA  DX, com_pret
  Mov  AH, 09h
  Int  21h
  ret
ComPret endp


;============================================================================
;  procedure GenererBreak
;    -entre: nant
;    -sortie: nant
;============================================================================
GenererBreak proc near
  Mov  DX, portxxB             ;-- Line Control Register
  Mov  AL, 01000000b           ;-- bit 6: generer Break
  Out  DX, AL

  LEA  DX, gen_break           ;-- afficher message Break envoye
  Mov  AH, 09h
  Int  21h
GB_Fin:
  ret
GenererBreak endp


;============================================================================
;  procedure RecevoirBreak
;    -entre: nant
;    -sortie: nant
;============================================================================
RecevoirBreak proc near
  LEA  DX, recep_break         ;-- afficher message Break recu
  Mov  AH, 09h
  Int  21h
RB_Fin:
  ret
RecevoirBreak endp

;============================================================================
;  procedure Ralentir (4 TESTING PURPOSE)
;    -entre: nant
;    -sortie: nant
;============================================================================
Ralentir proc near
  Mov  DI, 5000

R_BoucleDI:
  Mov  SI, 50000

R_BoucleSI:
  Nop
  Nop
  Dec  SI
  Jnz  R_BoucleSI
  Dec  DI
  Jnz  R_BoucleDI

  ret
Ralentir endp


;============================================================================
;  procedure RAZCompteurCar
;    -entre: nant
;    -sortie: nant
;============================================================================
RAZCompteurCar proc near
  Mov  nb_car_recu, 0        ;-- on ne peut plus supprimer
  Mov  nb_car_emis, 0         

  ret
RAZCompteurCar endp


;============================================================================
;  procedure MsgDistant
;    -entre: AL caractre reu
;    -sortie: nant
;============================================================================
MsgDistant proc near
  Mov  BL, AL
  Test presentation_plus, VRAI
  Jz   MsgDistant_Caractere  ;-- pas de demande de "prsentation +", on quitte

MsgDistant_CRLF:
  Cmp  AL, 13                ;-- CR reu ?
  Jne  MsgDistant_Backspace 

  LEA  DX, msg_distant       ;-- oui, on imprime CRLF et tiquette
  Mov  AH, 09
  Int  21h
  Mov  nb_car_recu, 0 ;-- RAZ compteur
  Jmp  MsgDistant_Fin

MsgDistant_Backspace:
  Cmp  AL, 8
  Jne  MsgDistant_Etiquette

  Cmp  nb_car_recu, 0
  Je   MsgDistant_Fin        ;-- on ne peut plus supprimer !
    
  Dec  nb_car_recu
  Mov  DL, AL
  Mov  AH, 02
  Int  21h                   ;-- imprimer 'backspace'
  Mov  DL, ' '               ;-- AH non modifi
  Int  21h                   ;-- imprimer 'espace'

  Dec  nb_car_recu           ;-- corriger le Inc automatique
                             ;-- de MsgDistant_Caractere
  Jmp  MsgDistant_Caractere  ;-- imprimer encore 'backspace' pour corriger
                             ;-- le dcalage automatique du curseur

MsgDistant_Etiquette:
  Test distant_en_cours, VRAI
  Jnz  MsgDistant_Caractere

  Mov  distant_en_cours, VRAI
  Mov  local_en_cours, FAUX
  Call RAZCompteurCar        ;-- on ne peut plus supprimer apres RAZ
  LEA  DX, msg_distant       ;-- afficher tiquette
  Mov  AH, 09
  Int  21h

MsgDistant_Caractere:  
  Inc  nb_car_recu
  Mov  DL, BL
  Mov  AH, 02
  Int  21h                   ;-- imprimer a l'ecran le caractre reu

MsgDistant_Fin: 
  ret
MsgDistant endp


;============================================================================
;  procedure MsgLocal
;    -entre: AL caractre envoy
;    -sortie: nant
;============================================================================
MsgLocal proc near
  Mov  BL, AL
  Test presentation_plus, VRAI
  Jz   MsgLocal_Fin      ;-- pas de demande de "prsentation +", on quitte

MsgLocal_CRLF:
  Cmp  AL, 13            ;-- CR envoy ?
  Jne  MsgLocal_backspace

  Mov  nb_car_emis, 0    ;-- RAZ compteur
  LEA  DX, msg_local
  Mov  AH, 09
  Int  21h
  Jmp  MsgLocal_Fin

MsgLocal_Backspace:
  Cmp  AL, 8             ;-- backspace envoy ?
  Jne  MsgLocal_Etiquette
  
  Cmp  nb_car_emis, 0
  Je   MsgLocal_Fin      ;-- on peut plus supprimer

  Dec  nb_car_emis
  Mov  AH, 02
  Mov  DL, 8
  Int  21h
  Mov  DL, ' '
  Int  21h
  Dec  nb_car_emis       ;-- elimine le Inc automatique de
                         ;-- MsgLocal_Afficher

  Jmp  MsgLocal_Afficher

MsgLocal_Etiquette:
  Test local_en_cours, VRAI
  Jnz  MsgLocal_Afficher

  Mov  local_en_cours, VRAI      ;-- basculer les tats
  Mov  distant_en_cours, FAUX
  Call RAZCompteurCar
  LEA  DX, msg_local
  Mov  AH, 09
  Int  21h

MsgLocal_Afficher:
  Inc  nb_car_emis
  Mov  DL, BL
  Mov  AH, 02
  Int  21h  

MsgLocal_Fin:
  ret
MsgLocal endp

;============================================================================
;  procedure Communiquer
;    -entre: nant
;    -sortie: nant
;============================================================================
Communiquer proc near
  Mov  portxxDLu, FAUX

C_Si_Donnee_Disponible:
  Test portxxDLU, VRAI
  Jnz  C_xxD_Deja_Lu       ;-- port deja lu par test THRE
                           ;-- son contenu est dispo. dans BL
  Mov  DX, portxxD
  In   AL, DX
  Mov  BL, AL              ;-- utilisons BL car AL modifie par Int 21h

C_xxD_Deja_Lu:
  Test BL, 00010000b       ;-- reception Break ?
  Jz   C_Si_Donnee_Prete
  Test control_break, VRAI ;-- controle de Break active ?
  Jz   C_Si_Donnee_Prete   ;-- non, on ignore le Break
  Call RecevoirBreak       ;-- Break recu
  Jmp  C_Fin               ;-- terminer la communication

C_Si_Donnee_Prete:
  Test BL, 00000001b       ;-- data ready ?
  Jz   C_Clavier           ;-- non, on verifie si une touche frappee ?

  Test BL, 00000010b       ;-- erreur de cadence ?
  Jz   C_Cadence_OK        ;-- non, test parite

  Mov  DX, err_cadence     ;-- imprimer message
  Mov  AH, 09h
  Int  21h
  Call RAZCompteurCar      ;-- eviter que l'user supprime le message
                           ;-- en appuyant sur backspace

C_Cadence_OK:              ;-- test de parite
  Test BL, 00000100b       ;-- erreur de parite ?
  Jz   C_Parite_OK

  Mov  DX, err_parite
  Mov  AH, 09h
  Int  21h
  Call RAZCompteurCar

C_Parite_OK:               ;-- test de trame
  Test BL, 00001000b       ;-- erreur de trame ?
  Jz   C_Tout_OK           ;-- aucune erreur de transmission
                           ;-- detectee, on va lire le caractere
                           ;-- recu
  Mov  DX, err_trame
  Mov  AH, 09h
  Int  21h
  Call RAZCompteurCar

C_Tout_OK:
  Mov  DX, COM
  In   AL, DX              ;-- oui on lit le caractere
  Call MsgDistant

;(4 TESTING PURPOSE)----   Call Ralentir            ;-- on boucle un peu pour tester FIFO 

C_Clavier:
;--------------------- ATTENTION ----------------------
; utilisation de l'int 16h ne conduit pas aux resultats
; souhaites, il faut imperativement utiliser
; les services 0Bh, et 07h de l'int 21h de DOS
; Pourquoi ? je ne le sais pas, le 4 Mars 2002
;------------------------------------------------------
;--  Mov  AH, 11h             ;-- service for enhanced keyboard
;--  Int  16h
;--  Jnz  C_Si_Donnee_Disponible

  Mov  AH, 0Bh
  Int  21h
  Test AL, 0FFh
  Jnz  C_Clavier_Lire_Touche
  Mov  portxxDLU, FAUX
  Jmp  C_Si_Donnee_Disponible

;--  Mov  AH, 10h             ;-- read / enhanced keyboard
;--  Int  16h                 ;-- lecture du caract dans memoire

C_Clavier_Lire_Touche:
  Mov  AH, 07h
  Int  21h
  Cmp  AL, 27              ;-- echap ?
  Jz   C_Envoyer_Break     ;-- oui on envoye BREAK sur la ligne
  Mov  portxxDLu, VRAI
  Mov  AH, AL              ;-- sauvegarde caractere dans AH

C_Si_THRE_Vide:
  Mov  DX, portxxD         ;-- Line Status Register
  In   AL, DX
  Mov  BL, AL
  Test BL, 00100000b       ;-- bit 5: THRE vide ?
  Jz   C_Si_THRE_Vide      ;-- non on boucle
                           ;-- risques de perte d'informations sur
                           ;-- la ligne quand on boucle

C_Envoyer_Char:            ;-- THRE vide, on envoie la sauce
  Mov  AL, AH              ;-- recupere caractere dans AH
  Mov  DX, COM
  Out  DX, AL              ;-- on l'envoie
  Mov  BH, BL              ;-- sauvegarder BL
  Call MsgLocal            
  Mov  BL, BH              ;-- restaurer BL
  Jmp  C_Si_Donnee_Disponible

C_Envoyer_Break:
  Test control_break, VRAI ;-- controle de Break active ?
  Jz   C_fin               ;-- non, on ne genere pas le break
  Call GenererBreak

C_fin:
  LEA  DX, com_fin
  Mov  AH, 09h
  Int  21h
  ret
Communiquer endp


;============================================================================
;  procedure ConfigActuelle
;    -entre: nant
;    -sortie: nant
;============================================================================
ConfigActuelle proc near
  Mov  AX, DS
  Mov  ES, AX
  Mov  AH, 02              ;-- aller  la ligne
  Mov  DL, 10
  Int  21h
  Mov  DL, 13
  Int  21h
  Jmp  ConfigAct_COM

ConfigAct_Imprimer:        ;-- sous-programme interne d'affichage
  Mov  BX, DX
  Mov  AX, CX
  Mul  AH                  ;-- dterminer le dcalage par rapport
                           ;-- au dbut du message  

  Add  DX, AX              ;-- pointe sur le dbut du message
  Mov  AX, CX
  Inc  AH
  Mul  AH
  
  Add  BX, AX                  ;-- BX pointe sur le message suivant
  Mov  byte ptr ES:[BX+2], '$' ;-- crase l'octer par '$'
  Add  DX, 5                   ;-- dcale DX pour avoir le premier caractre
  Mov  AH, 09
  Int  21h                     ;-- imprimer le message
  Mov  AH, CH
  Add  AH, '2'
  Mov  byte ptr ES:[BX+2], AH    ;-- restaurer l'octet cras

  ret

ConfigAct_On:                  ;-- sous-programme interne d'affichage
  LEA  DX, msg_on
  Mov  AH, 09
  Int  21h
  ret

ConfigAct_Off:                 ;-- sous-programme interne d'affichage
  LEA  DX, msg_off
  Mov  AH, 09
  Int  21h
  ret

ConfigAct_COM:                 ;-- message du port COM
  Mov  CH, val_COM
  Mov  CL, len_COM
  LEA  DX, label_COM
  Call ConfigAct_Imprimer

ConfigAct_Vitesse:             ;-- message de la vitesse
  Mov  CH, val_vitesse
  Mov  CL, len_vitesse
  LEA  DX, label_vitesse
  Call ConfigAct_Imprimer

ConfigAct_Trame:               ;-- message de la trame
  Mov  CH, val_long
  Mov  CL, len_long
  LEA  DX, label_long  
  Call ConfigAct_Imprimer

ConfigAct_Parite:              ;-- message de la parit
  Mov  CH, val_parite
  Mov  CL, len_parite
  LEA  DX, label_parite  
  Call ConfigAct_Imprimer

ConfigAct_FIFO:                ;-- message de l'tat du FIFO
  LEA  DX, msg_FIFO
  Mov  AH, 09
  Int  21h
  Test active_FIFO, VRAI
  Jnz  ConfigAct_FIFO_on

  Call ConfigAct_Off
  Jmp  ConfigAct_Break

ConfigAct_FIFO_on:
  Call ConfigAct_On

ConfigAct_Break:               ;-- message de l'tat du BREAK
  LEA  DX, msg_Break
  Mov  AH, 09
  Int  21h
  Test control_Break, VRAI
  Jnz  ConfigAct_Break_on

  Call ConfigAct_Off
  Jmp  ConfigAct_Loopback

ConfigAct_Break_on:
  Call ConfigAct_On

ConfigAct_Loopback:            ;-- message de l'tat du loopback
  LEA  DX, msg_Loopback
  Mov  AH, 09
  Int  21h
  Test active_loopB, VRAI
  Jnz  ConfigAct_Loopback_on

  Call ConfigAct_Off
  Jmp  ConfigAct_Presentation

ConfigAct_Loopback_on:
  Call ConfigAct_On

ConfigAct_Presentation:       ;-- message de l'tat de la "prsentation plus"
  LEA  DX, msg_presentation
  Mov  AH, 09
  Int  21h
  Test presentation_plus, VRAI
  Jnz  ConfigAct_PrePlus_on

  Call ConfigAct_Off
  Jmp  ConfigAct_Fin

ConfigAct_PrePlus_on:
  Call ConfigAct_On  

ConfigAct_Fin:
  ret
ConfigActuelle endp


;============================================================================
;  SOUS PROGRAMMES TRAITANT DES CHOIX UTILISATEURS
;============================================================================

;============================================================================
;  procedure GoGoGo
;    -entre: nant
;    -sortie: nant
;============================================================================
GoGoGo proc near
  Mov  AX, COM          ;-- calcul relatif des adresses de port
  Inc  AX
  Mov  portxx9, AX
  Inc  AX
  Mov  portxxA, AX
  Inc  AX
  Mov  portxxB, AX
  Inc  AX
  Mov  portxxC, AX
  Inc  AX
  Mov  portxxD, AX

  Mov  distant_en_cours, FAUX
  Mov  local_en_cours, FAUX
  Call RAZCompteurCar

  Call DesactiverIRQ  ;(4 TESTING PURPOSE)  
  Call EngagerBaud
  Call ConfigurerPaquet
  Call ActiverFIFO

  Call ConfigActuelle
  Call ComPret
  Call Communiquer

  ret
GoGoGo endp


;============================================================================
;  procedure MenuCOM
;============================================================================
MenuCOM proc near
  Mov  AH, 09h
  LEA  DX, choix_COM
  Int  21h

  Mov  AH, 01h
  Int  21h
  Cmp  AL, '1'
  Jne  MenuCOM_COM2
  Mov  COM, COM1
  Mov  val_COM, 0           ;-- pour affichage plus tard
  Jmp  MenuCOM_Fin

MenuCOM_COM2:
  Cmp  AL, '2'
  Jne  MenuCOM_Choix_Inconnu
  Mov  COM, COM2
  Mov  val_COM, 1           ;-- pour affichage plus tard  
  Jmp  MenuCOM_Fin

MenuCOM_Choix_Inconnu:
  Mov  AH, 09h
  LEA  DX, err_choix
  Int  21h

MenuCOM_Fin:
  ret
MenuCOM endp



;============================================================================
;  procedure MenuVitesse
;============================================================================
MenuVitesse proc near
  Mov  AH, 09h
  LEA  DX, choix_vitesse           ;-- affichage menu
  Int  21h

  Mov  AH, 01h
  Int  21h
  Cmp  AL, '1'
  Jb   MenuVitesse_Choix_Inconnu
  Cmp  AL, '8'
  Ja   MenuVitesse_Choix_Inconnu

  Sub  AL, '1'                     ;-- Convertit AL en chiffre - 1
  Mov  val_vitesse, AL             ;-- sauvegarder pour afficher plus tard
  Mov  CL, AL
  Xor  AH, AH
  Mov  AL, valeur_ref
  Shr  AL, CL                      ;-- divise AL par 2 fois CL
  ADC  AL, 0                       ;-- corriger les valeurs extrmes
                                   ;-- cad. 57600 & 115200 bauds
  Mov  vitesse_COM, AL             ;-- affecter valeur dans variable
                                   ;-- globale vitesse_COM
  Jmp  MenuVitesse_Fin

MenuVitesse_Choix_Inconnu:
  Mov  AH, 09h
  LEA  DX, err_choix
  Int  21h
MenuVitesse_Fin:
  ret
MenuVitesse endp


;============================================================================
;  procedure MenuLongueurMot
;============================================================================
MenuLongueurMot proc near
  Mov  AH, 09h
  LEA  DX, choix_long
  Int  21h

  Mov  AH, 01h
  Int  21h
  Cmp  AL, '1'
  Jb   MLM_Choix_Inconnu
  Cmp  AL, '8'
  Ja   MLM_Choix_Inconnu
  Sub  AL, '1'
  Mov  config_trame, AL
  Mov  val_long, AL           ;-- sauvegarder pour affichage
  Jmp  MLM_Fin

MLM_Choix_Inconnu:
  Mov  AH, 09h
  LEA  DX, err_choix
  Int  21h

MLM_Fin:
  ret
MenuLongueurMot endp


;============================================================================
;  procedure MenuParite
;============================================================================
MenuParite proc near
  Mov  AH, 09h
  LEA  DX, choix_parite
  Int  21h

  Mov  BL, config_trame             ;-- BL prendre la config. actuelle
  Mov  AH, 01h
  Int  21h
  Cmp  AL, '1'
  Jne  MenuParite_Odd
  And  BL, 11000111b                ;-- met bit 3, 4, 5 a 0
  Mov  val_parite, 0                ;-- pour affichage
  Jmp  MenuParite_Fin               ;-- parite desactive par oper. AND

MenuParite_Odd:
  Cmp  AL, '2'
  Jne  MenuParite_Even
  And  BL, 11000111b                ;-- met bit 3, 4, 5 a 0
  Or   BL, 00001000b                ;-- active parite impaire
  Mov  val_parite, 1                ;-- pour affichage
  Jmp  MenuParite_Fin

MenuParite_Even:
  Cmp  AL, '3'
  Jne  MenuParite_Choix_Inconnu
  And  BL, 11000111b                ;-- met bit 3, 4, 5 a 0
  Or   BL, 00011000b                ;-- active parite paire
  Mov  val_parite, 2                ;-- pour affichage
  Jmp  MenuParite_Fin

MenuParite_Choix_Inconnu:
  Mov  AH, 09h
  LEA  DX, err_choix
  Int  21h

MenuParite_Fin:
  Mov  config_trame, BL
  ret
MenuParite endp


;============================================================================
;  procedure MenuLoopBack
;============================================================================
MenuLoopBack proc near
  Mov  AH, 09
  LEA  DX, choix_loopb
  Int  21h

  Mov  AH, 01
  Int  21h
  Cmp  AL, '1'
  Jne  MLB_active
  Mov  active_loopb, FAUX
  Jmp  MLB_Fin

MLB_active:
  Cmp  AL, '2'
  Jne  MLB_Choix_Inconnu
  Mov  active_loopb, VRAI
  Jmp  MLB_Fin

MLB_Choix_Inconnu:
  Mov  AH, 09h
  LEA  DX, err_choix
  Int  21h

MLB_Fin:
  ret
MenuLoopBack endp


;============================================================================
;  procedure MenuFIFO
;============================================================================
MenuFIFO proc near
  Mov  AH, 09
  LEA  DX, choix_FIFO
  Int  21h

  Mov  AH, 01
  Int  21h
  Cmp  AL, '1'
  Jne  MFIFO_Desactive
  Mov  active_FIFO, VRAI
  Jmp  MFIFO_Fin

MFIFO_Desactive:
  Cmp  AL, '2'
  Jne  MFIFO_Choix_Inconnu
  Mov  active_FIFO, FAUX
  Jmp  MFIFO_Fin

MFIFO_Choix_Inconnu:
  Mov  AH, 09h
  LEA  DX, err_choix
  Int  21h

MFIFO_Fin:
  ret
MenuFIFO endp


;============================================================================
;  procedure MenuBREAK
;============================================================================
MenuBreak proc near
  Mov  AH, 09h
  LEA  DX, choix_break
  Int  21h

  Mov  AH, 01h
  Int  21h
  Cmp  AL, '1'
  Jne  MenuBreak_Faux
  Mov  control_break, VRAI
  Jmp  MenuBreak_Fin

MenuBreak_Faux:
  Cmp  AL, '2'
  Jne  MenuBreak_Choix_Inconnu
  Mov  control_break, FAUX
  Jmp  MenuBreak_Fin

MenuBreak_Choix_Inconnu:
  Mov  AH, 09h
  LEA  DX, err_choix
  Int  21h

MenuBreak_Fin:
  ret
MenuBreak endp


;============================================================================
;  procedure MenuPresentation
;============================================================================
MenuPresentation proc near
  Mov  AH, 09h
  LEA  DX, choix_presentation
  Int  21h

  Mov  AH, 01h
  Int  21h
  Cmp  AL, '1'
  Jne  MenuPresentation_Faux

  Mov  presentation_plus, VRAI
  LEA  DX, msg_cadence
  Mov  err_cadence, DX
  LEA  DX, msg_trame
  Mov  err_trame, DX
  LEA  DX, msg_parite
  Mov  err_parite, DX 
  Jmp  MenuPresentation_Fin

MenuPresentation_Faux:
  Cmp  AL, '2'
  Jne  MenuPresentation_Choix_Inconnu

  Mov  presentation_plus, FAUX
  LEA  DX, car_cadence
  Mov  err_cadence, DX
  LEA  DX, car_trame
  Mov  err_trame, DX
  LEA  DX, car_parite
  Mov  err_parite, DX   
  Jmp  MenuPresentation_Fin

MenuPresentation_Choix_Inconnu:
  Mov  AH, 09h
  LEA  DX, err_choix
  Int  21h

MenuPresentation_Fin:
  ret
MenuPresentation endp


;============================================================================
;  procedure MenuConfigurerLigne
;============================================================================
MenuConfigurerLigne proc near
   Mov  AH, 09h
   LEA  DX, choix_config
   Int  21h

   Mov  AH, 01h
   Int  21h

   Cmp  AL, '1'
   Jne  MCL_Vitesse
   Call MenuCOM
   Jmp  MCL_Fin

MCL_Vitesse:
   Cmp  AL, '2'
   Jne  MCL_Longueur_Mot
   Call MenuVitesse
   Jmp  MCL_Fin

MCL_Longueur_Mot:
   Cmp  AL, '3'
   Jne  MCL_Parite
   Call MenuLongueurMot
   Jmp  MCL_Fin

MCL_Parite:
   Cmp  AL, '4'
   Jne  MCL_Stop_Bit
   Call MenuParite
   Jmp  MCL_Fin

MCL_Stop_Bit:
   Cmp  AL, '5'
   Jne  MCL_FIFO
   Call MenuLoopBack
   Jmp  MCL_Fin

MCL_FIFO:
   Cmp  AL, '6'
   Jne  MCL_Test_Break
   Call MenuFIFO
   Jmp  MCL_Fin

MCL_Test_Break:
   Cmp  AL, '7'
   Jne  MCL_Presentation
   Call MenuBreak
   Jmp  MCL_Fin

MCL_Presentation:
   Cmp  AL, '8'
   Jne  MCL_Choix_Inconnu
   Call MenuPresentation
   Jmp  MCL_Fin

MCL_Choix_Inconnu:
   Mov  AH, 09h
   LEA  DX, err_choix
   Int  21h

MCL_Fin:
   ret
MenuConfigurerLigne endp


;============================================================================
;  procedure MenuPrincipal
;============================================================================
MenuPrincipal proc near
MP_Debut:
  Mov  AH, 09h
  LEA  DX, choix_menu
  Int  21h

  Mov  AH, 01h
  Int  21h
  Cmp  AL, '1'                ;-- configurer la ligne
  Jne  MP_Config_Act

MP_Config_Ligne:
  Call MenuConfigurerLigne
  Jmp  MP_Debut

MP_Config_Act:
  Cmp  AL, '2'                ;-- afficher la configuration actuelle
  Jne  MP_Choix_Gogogo
  Call ConfigActuelle
  Jmp  MP_Debut

MP_Choix_Gogogo:
  Cmp  AL, '3'
  Jne  MP_Choix_Quitter

  Call GoGoGo                 ;-- lancer la communication
  Jmp  MP_Debut

MP_Choix_Quitter:
  Cmp  AL, '0'
  Jne  MP_Debut

  ret                         ;-- Quitter
MenuPrincipal endp


;============================================================================
;  PROGRAMME PRINCIPAL
;============================================================================

debut:
  Call MenuPrincipal

fin:
  Mov  AX, 4C00h
  Int  21H

Ends _CODE
End introduction
